@@ -53,9 +53,9 @@ static int oqsx_key_recreate_classickey(OQSX_KEY *key, oqsx_key_op_t op);
 ///// OQS_TEMPLATE_FRAGMENT_OQSNAMES_START
 
 #ifdef OQS_KEM_ENCODERS
-#define NID_TABLE_LEN 110
+#define NID_TABLE_LEN 93
 #else
-#define NID_TABLE_LEN 57
+#define NID_TABLE_LEN 58
 #endif
 
 static oqs_nid_name_t nid_names[NID_TABLE_LEN] = {
@@ -88,16 +88,6 @@ static oqs_nid_name_t nid_names[NID_TABLE_LEN] = {
      0},
     {0, "p521_frodo1344shake", OQS_KEM_alg_frodokem_1344_shake,
      KEY_TYPE_ECP_HYB_KEM, 256, 0},
-    {0, "kyber512", OQS_KEM_alg_kyber_512, KEY_TYPE_KEM, 128, 0},
-    {0, "p256_kyber512", OQS_KEM_alg_kyber_512, KEY_TYPE_ECP_HYB_KEM, 128, 0},
-    {0, "x25519_kyber512", OQS_KEM_alg_kyber_512, KEY_TYPE_ECX_HYB_KEM, 128, 0},
-    {0, "kyber768", OQS_KEM_alg_kyber_768, KEY_TYPE_KEM, 192, 0},
-    {0, "p384_kyber768", OQS_KEM_alg_kyber_768, KEY_TYPE_ECP_HYB_KEM, 192, 0},
-    {0, "x448_kyber768", OQS_KEM_alg_kyber_768, KEY_TYPE_ECX_HYB_KEM, 192, 0},
-    {0, "x25519_kyber768", OQS_KEM_alg_kyber_768, KEY_TYPE_ECX_HYB_KEM, 192, 0},
-    {0, "p256_kyber768", OQS_KEM_alg_kyber_768, KEY_TYPE_ECP_HYB_KEM, 192, 0},
-    {0, "kyber1024", OQS_KEM_alg_kyber_1024, KEY_TYPE_KEM, 256, 0},
-    {0, "p521_kyber1024", OQS_KEM_alg_kyber_1024, KEY_TYPE_ECP_HYB_KEM, 256, 0},
     {0, "mlkem512", OQS_KEM_alg_ml_kem_512, KEY_TYPE_KEM, 128, 0},
     {0, "p256_mlkem512", OQS_KEM_alg_ml_kem_512, KEY_TYPE_ECP_HYB_KEM, 128, 0},
     {0, "x25519_mlkem512", OQS_KEM_alg_ml_kem_512, KEY_TYPE_ECX_HYB_KEM, 128,
@@ -111,8 +101,8 @@ static oqs_nid_name_t nid_names[NID_TABLE_LEN] = {
     {0, "mlkem1024", OQS_KEM_alg_ml_kem_1024, KEY_TYPE_KEM, 256, 0},
     {0, "p521_mlkem1024", OQS_KEM_alg_ml_kem_1024, KEY_TYPE_ECP_HYB_KEM, 256,
      0},
-    {0, "p384_mlkem1024", OQS_KEM_alg_ml_kem_1024, KEY_TYPE_ECP_HYB_KEM, 256,
-     0},
+    {0, "SecP384r1MLKEM1024", OQS_KEM_alg_ml_kem_1024, KEY_TYPE_ECP_HYB_KEM,
+     256, 0},
     {0, "bikel1", OQS_KEM_alg_bike_l1, KEY_TYPE_KEM, 128, 0},
     {0, "p256_bikel1", OQS_KEM_alg_bike_l1, KEY_TYPE_ECP_HYB_KEM, 128, 0},
     {0, "x25519_bikel1", OQS_KEM_alg_bike_l1, KEY_TYPE_ECX_HYB_KEM, 128, 0},
@@ -121,23 +111,8 @@ static oqs_nid_name_t nid_names[NID_TABLE_LEN] = {
     {0, "x448_bikel3", OQS_KEM_alg_bike_l3, KEY_TYPE_ECX_HYB_KEM, 192, 0},
     {0, "bikel5", OQS_KEM_alg_bike_l5, KEY_TYPE_KEM, 256, 0},
     {0, "p521_bikel5", OQS_KEM_alg_bike_l5, KEY_TYPE_ECP_HYB_KEM, 256, 0},
-    {0, "hqc128", OQS_KEM_alg_hqc_128, KEY_TYPE_KEM, 128, 0},
-    {0, "p256_hqc128", OQS_KEM_alg_hqc_128, KEY_TYPE_ECP_HYB_KEM, 128, 0},
-    {0, "x25519_hqc128", OQS_KEM_alg_hqc_128, KEY_TYPE_ECX_HYB_KEM, 128, 0},
-    {0, "hqc192", OQS_KEM_alg_hqc_192, KEY_TYPE_KEM, 192, 0},
-    {0, "p384_hqc192", OQS_KEM_alg_hqc_192, KEY_TYPE_ECP_HYB_KEM, 192, 0},
-    {0, "x448_hqc192", OQS_KEM_alg_hqc_192, KEY_TYPE_ECX_HYB_KEM, 192, 0},
-    {0, "hqc256", OQS_KEM_alg_hqc_256, KEY_TYPE_KEM, 256, 0},
-    {0, "p521_hqc256", OQS_KEM_alg_hqc_256, KEY_TYPE_ECP_HYB_KEM, 256, 0},
 
 #endif /* OQS_KEM_ENCODERS */
-    {0, "dilithium2", OQS_SIG_alg_dilithium_2, KEY_TYPE_SIG, 128},
-    {0, "p256_dilithium2", OQS_SIG_alg_dilithium_2, KEY_TYPE_HYB_SIG, 128},
-    {0, "rsa3072_dilithium2", OQS_SIG_alg_dilithium_2, KEY_TYPE_HYB_SIG, 128},
-    {0, "dilithium3", OQS_SIG_alg_dilithium_3, KEY_TYPE_SIG, 192},
-    {0, "p384_dilithium3", OQS_SIG_alg_dilithium_3, KEY_TYPE_HYB_SIG, 192},
-    {0, "dilithium5", OQS_SIG_alg_dilithium_5, KEY_TYPE_SIG, 256},
-    {0, "p521_dilithium5", OQS_SIG_alg_dilithium_5, KEY_TYPE_HYB_SIG, 256},
     {0, "mldsa44", OQS_SIG_alg_ml_dsa_44, KEY_TYPE_SIG, 128},
     {0, "p256_mldsa44", OQS_SIG_alg_ml_dsa_44, KEY_TYPE_HYB_SIG, 128},
     {0, "rsa3072_mldsa44", OQS_SIG_alg_ml_dsa_44, KEY_TYPE_HYB_SIG, 128},
@@ -203,6 +178,16 @@ static oqs_nid_name_t nid_names[NID_TABLE_LEN] = {
     {0, "p521_mayo5", OQS_SIG_alg_mayo_5, KEY_TYPE_HYB_SIG, 256},
     {0, "CROSSrsdp128balanced", OQS_SIG_alg_cross_rsdp_128_balanced,
      KEY_TYPE_SIG, 128},
+    {0, "OV_Is_pkc", OQS_SIG_alg_uov_ov_Is_pkc, KEY_TYPE_SIG, 128},
+    {0, "p256_OV_Is_pkc", OQS_SIG_alg_uov_ov_Is_pkc, KEY_TYPE_HYB_SIG, 128},
+    {0, "OV_Ip_pkc", OQS_SIG_alg_uov_ov_Ip_pkc, KEY_TYPE_SIG, 128},
+    {0, "p256_OV_Ip_pkc", OQS_SIG_alg_uov_ov_Ip_pkc, KEY_TYPE_HYB_SIG, 128},
+    {0, "OV_Is_pkc_skc", OQS_SIG_alg_uov_ov_Is_pkc_skc, KEY_TYPE_SIG, 128},
+    {0, "p256_OV_Is_pkc_skc", OQS_SIG_alg_uov_ov_Is_pkc_skc, KEY_TYPE_HYB_SIG,
+     128},
+    {0, "OV_Ip_pkc_skc", OQS_SIG_alg_uov_ov_Ip_pkc_skc, KEY_TYPE_SIG, 128},
+    {0, "p256_OV_Ip_pkc_skc", OQS_SIG_alg_uov_ov_Ip_pkc_skc, KEY_TYPE_HYB_SIG,
+     128},
     ///// OQS_TEMPLATE_FRAGMENT_OQSNAMES_END
 };
 
@@ -543,9 +528,8 @@ static const OQSX_EVP_INFO nids_sig[] = {
 
 };
 // These two array need to stay synced:
-// note only leading 4 chars of alg name are checked
-static const char *OQSX_ECP_NAMES[] = {
-    "p256", "p384", "p521", "SecP256r1", "SecP384r1", "SecP521r1", 0};
+static const char *OQSX_ECP_NAMES[] = {"p256",      "p384",      "p521",
+                                       "SecP256r1", "SecP384r1", "SecP521r1"};
 static const OQSX_EVP_INFO nids_ecp[] = {
     {EVP_PKEY_EC, NID_X9_62_prime256v1, 0, 65, 121, 32, 0}, // 128 bit
     {EVP_PKEY_EC, NID_secp384r1, 0, 97, 167, 48, 0},        // 192 bit
@@ -553,22 +537,17 @@ static const OQSX_EVP_INFO nids_ecp[] = {
     {EVP_PKEY_EC, NID_X9_62_prime256v1, 0, 65, 121, 32, 0}, // 128 bit
     {EVP_PKEY_EC, NID_secp384r1, 0, 97, 167, 48, 0},        // 192 bit
     {EVP_PKEY_EC, NID_secp521r1, 0, 133, 223, 66, 0},       // 256 bit
-    {0, 0, 0, 0, 0, 0, 0}                                   // 256 bit
 };
 
 // These two array need to stay synced:
-// note only leading 4 chars of alg name are checked
-static const char *OQSX_ECX_NAMES[] = {"x25519", "x448", "X25519", "X448", 0};
+static const char *OQSX_ECX_NAMES[] = {"x25519", "x448"};
 static const OQSX_EVP_INFO nids_ecx[] = {
     {EVP_PKEY_X25519, 0, 1, 32, 32, 32, 0}, // 128 bit
     {EVP_PKEY_X448, 0, 1, 56, 56, 56, 0},   // 192 bit
-    {EVP_PKEY_X25519, 0, 1, 32, 32, 32, 0}, // 128 bit
-    {EVP_PKEY_X448, 0, 1, 56, 56, 56, 0},   // 192 bit
-    {0, 0, 0, 0, 0, 0, 0}                   // 256 bit
 };
 
 static int oqsx_hybsig_init(int bit_security, OQSX_EVP_CTX *evp_ctx,
-                            char *algname) {
+                            OSSL_LIB_CTX *libctx, char *algname) {
     int ret = 1;
     int idx = (bit_security - 128) / 64;
     ON_ERR_GOTO(idx < 0 || idx > 5, err_init);
@@ -600,12 +579,14 @@ static int oqsx_hybsig_init(int bit_security, OQSX_EVP_CTX *evp_ctx,
         ret = EVP_PKEY_set_type(evp_ctx->keyParam, evp_ctx->evp_info->keytype);
         ON_ERR_SET_GOTO(ret <= 0, ret, -1, err_init);
 
-        evp_ctx->ctx = EVP_PKEY_CTX_new(evp_ctx->keyParam, NULL);
+        evp_ctx->ctx =
+            EVP_PKEY_CTX_new_from_pkey(libctx, evp_ctx->keyParam, NULL);
         ON_ERR_SET_GOTO(!evp_ctx->ctx, ret, -1, err_init);
     } else {
         evp_ctx->evp_info = &nids_sig[idx];
 
-        evp_ctx->ctx = EVP_PKEY_CTX_new_id(evp_ctx->evp_info->keytype, NULL);
+        evp_ctx->ctx = EVP_PKEY_CTX_new_from_name(
+            libctx, OBJ_nid2sn(evp_ctx->evp_info->keytype), NULL);
         ON_ERR_GOTO(!evp_ctx->ctx, err_init);
 
         if (idx < 5) { // EC
@@ -631,20 +612,27 @@ err_init:
     return ret;
 }
 
-static const int oqshybkem_init_ecp(char *tls_name, OQSX_EVP_CTX *evp_ctx) {
+static const int oqshybkem_init_ecp(char *tls_name, OQSX_EVP_CTX *evp_ctx,
+                                    OSSL_LIB_CTX *libctx) {
     int ret = 1;
-    int idx = 0;
+    const OQSX_EVP_INFO *evp_info = NULL;
 
-    while (idx < OSSL_NELEM(OQSX_ECP_NAMES)) {
-        if (!strncmp(tls_name, OQSX_ECP_NAMES[idx], (idx < 3) ? 4 : 7))
+    for (int i = 0; i < OSSL_NELEM(OQSX_ECP_NAMES); i++) {
+        if (!strncasecmp(tls_name, OQSX_ECP_NAMES[i],
+                         strlen(OQSX_ECP_NAMES[i]))) {
+            evp_info = &nids_ecp[i];
             break;
-        idx++;
+        }
+    }
+    if (evp_info == NULL) {
+        OQS_KEY_PRINTF2("OQS KEY: Incorrect P hybrid KEM name: %s\n", tls_name);
+        goto err_init_ecp;
     }
-    ON_ERR_GOTO(idx < 0 || idx > 6, err_init_ecp);
 
-    evp_ctx->evp_info = &nids_ecp[idx];
+    evp_ctx->evp_info = evp_info;
 
-    evp_ctx->ctx = EVP_PKEY_CTX_new_id(evp_ctx->evp_info->keytype, NULL);
+    evp_ctx->ctx = EVP_PKEY_CTX_new_from_name(
+        libctx, OBJ_nid2sn(evp_ctx->evp_info->keytype), NULL);
     ON_ERR_GOTO(!evp_ctx->ctx, err_init_ecp);
 
     ret = EVP_PKEY_paramgen_init(evp_ctx->ctx);
@@ -661,18 +649,24 @@ err_init_ecp:
     return ret;
 }
 
-static const int oqshybkem_init_ecx(char *tls_name, OQSX_EVP_CTX *evp_ctx) {
+static const int oqshybkem_init_ecx(char *tls_name, OQSX_EVP_CTX *evp_ctx,
+                                    OSSL_LIB_CTX *libctx) {
     int ret = 1;
-    int idx = 0;
+    const OQSX_EVP_INFO *evp_info = NULL;
 
-    while (idx < OSSL_NELEM(OQSX_ECX_NAMES)) {
-        if (!strncmp(tls_name, OQSX_ECX_NAMES[idx], 4))
+    for (int i = 0; i < OSSL_NELEM(OQSX_ECX_NAMES); i++) {
+        if (!strncasecmp(tls_name, OQSX_ECX_NAMES[i],
+                         strlen(OQSX_ECX_NAMES[i]))) {
+            evp_info = &nids_ecx[i];
             break;
-        idx++;
+        }
+    }
+    if (evp_info == NULL) {
+        OQS_KEY_PRINTF2("OQS KEY: Incorrect X hybrid KEM name: %s\n", tls_name);
+        goto err_init_ecx;
     }
-    ON_ERR_GOTO(idx < 0 || idx > 4, err_init_ecx);
 
-    evp_ctx->evp_info = &nids_ecx[idx];
+    evp_ctx->evp_info = evp_info;
 
     evp_ctx->keyParam = EVP_PKEY_new();
     ON_ERR_SET_GOTO(!evp_ctx->keyParam, ret, -1, err_init_ecx);
@@ -680,7 +674,7 @@ static const int oqshybkem_init_ecx(char *tls_name, OQSX_EVP_CTX *evp_ctx) {
     ret = EVP_PKEY_set_type(evp_ctx->keyParam, evp_ctx->evp_info->keytype);
     ON_ERR_SET_GOTO(ret <= 0, ret, -1, err_init_ecx);
 
-    evp_ctx->ctx = EVP_PKEY_CTX_new(evp_ctx->keyParam, NULL);
+    evp_ctx->ctx = EVP_PKEY_CTX_new_from_pkey(libctx, evp_ctx->keyParam, NULL);
     ON_ERR_SET_GOTO(!evp_ctx->ctx, ret, -1, err_init_ecx);
 
 err_init_ecx:
@@ -986,9 +980,10 @@ static int oqsx_key_recreate_classickey(OQSX_KEY *key, oqsx_key_op_t op) {
                                 ->keytype,
                             &npk, &enc_pubkey, key->pubkeylen_cmp[i]);
                     } else
-                        key->classical_pkey = EVP_PKEY_new_raw_public_key(
-                            key->oqsx_provider_ctx.oqsx_evp_ctx->evp_info
-                                ->keytype,
+                        key->classical_pkey = EVP_PKEY_new_raw_public_key_ex(
+                            key->libctx,
+                            OBJ_nid2sn(key->oqsx_provider_ctx.oqsx_evp_ctx
+                                           ->evp_info->keytype),
                             NULL, enc_pubkey, key->pubkeylen_cmp[i]);
                     if (!key->classical_pkey) {
                         ERR_raise(ERR_LIB_USER, OQSPROV_R_INVALID_ENCODING);
@@ -1013,14 +1008,16 @@ static int oqsx_key_recreate_classickey(OQSX_KEY *key, oqsx_key_op_t op) {
                     if (!key->oqsx_provider_ctx.oqsx_evp_ctx->evp_info
                              ->raw_key_support) {
                         EVP_PKEY *npk;
-                        key->classical_pkey = d2i_PrivateKey(
+                        key->classical_pkey = d2i_PrivateKey_ex(
                             key->oqsx_provider_ctx.oqsx_evp_ctx->evp_info
                                 ->keytype,
-                            NULL, &enc_privkey, key->privkeylen_cmp[i]);
+                            NULL, &enc_privkey, key->privkeylen_cmp[i],
+                            key->libctx, NULL);
                     } else {
-                        key->classical_pkey = EVP_PKEY_new_raw_private_key(
-                            key->oqsx_provider_ctx.oqsx_evp_ctx->evp_info
-                                ->keytype,
+                        key->classical_pkey = EVP_PKEY_new_raw_private_key_ex(
+                            key->libctx,
+                            OBJ_nid2sn(key->oqsx_provider_ctx.oqsx_evp_ctx
+                                           ->evp_info->keytype),
                             NULL, enc_privkey, key->privkeylen_cmp[i]);
                     }
                     if (!key->classical_pkey) {
@@ -1073,9 +1070,9 @@ static int oqsx_key_recreate_classickey(OQSX_KEY *key, oqsx_key_op_t op) {
                     goto rec_err;
                 }
                 if (key->evp_info->raw_key_support) {
-                    key->classical_pkey = EVP_PKEY_new_raw_public_key(
-                        key->evp_info->keytype, NULL, enc_pubkey,
-                        classical_pubkey_len);
+                    key->classical_pkey = EVP_PKEY_new_raw_public_key_ex(
+                        key->libctx, OBJ_nid2sn(key->evp_info->keytype), NULL,
+                        enc_pubkey, classical_pubkey_len);
                     if (!key->classical_pkey) {
                         ERR_raise(ERR_LIB_USER, OQSPROV_R_INVALID_ENCODING);
                         goto rec_err;
@@ -1105,9 +1102,9 @@ static int oqsx_key_recreate_classickey(OQSX_KEY *key, oqsx_key_op_t op) {
                     key->comp_privkey[idx_classic];
                 unsigned char *enc_pubkey = key->comp_pubkey[idx_classic];
                 if (key->evp_info->raw_key_support) {
-                    key->classical_pkey = EVP_PKEY_new_raw_private_key(
-                        key->evp_info->keytype, NULL, enc_privkey,
-                        classical_privkey_len);
+                    key->classical_pkey = EVP_PKEY_new_raw_private_key_ex(
+                        key->libctx, OBJ_nid2sn(key->evp_info->keytype), NULL,
+                        enc_privkey, classical_privkey_len);
                     if (!key->classical_pkey) {
                         ERR_raise(ERR_LIB_USER, OQSPROV_R_INVALID_ENCODING);
                         goto rec_err;
@@ -1127,9 +1124,9 @@ static int oqsx_key_recreate_classickey(OQSX_KEY *key, oqsx_key_op_t op) {
                     }
 #endif
                 } else {
-                    key->classical_pkey =
-                        d2i_PrivateKey(key->evp_info->keytype, NULL,
-                                       &enc_privkey, classical_privkey_len);
+                    key->classical_pkey = d2i_PrivateKey_ex(
+                        key->evp_info->keytype, NULL, &enc_privkey,
+                        classical_privkey_len, key->libctx, NULL);
                     if (!key->classical_pkey) {
                         ERR_raise(ERR_LIB_USER, OQSPROV_R_INVALID_ENCODING);
                         goto rec_err;
@@ -1312,15 +1309,15 @@ OQSX_KEY *oqsx_key_from_pkcs8(const PKCS8_PRIV_KEY_INFO *p8inf,
                                 return NULL;
                             }
                             ec_pkey = EVP_PKEY_new();
-                            d2i_PrivateKey(
-                                EVP_PKEY_EC, &ec_pkey, &buf3,
-                                aType->value.sequence->length); // create
-                                                                // a new
-                                                                // EVP_PKEY
-                                                                // using
-                                                                // ec
-                                                                // priv
-                                                                // key
+                            d2i_PrivateKey_ex(EVP_PKEY_EC, &ec_pkey, &buf3,
+                                              aType->value.sequence->length,
+                                              libctx, NULL); // create
+                                                             // a new
+                                                             // EVP_PKEY
+                                                             // using
+                                                             // ec
+                                                             // priv
+                                                             // key
 
                             // set parameters for the
                             // new priv key format
@@ -1419,7 +1416,7 @@ OQSX_KEY *oqsx_key_from_pkcs8(const PKCS8_PRIV_KEY_INFO *p8inf,
     return oqsx;
 }
 
-static const int (*init_kex_fun[])(char *, OQSX_EVP_CTX *) = {
+static const int (*init_kex_fun[])(char *, OQSX_EVP_CTX *, OSSL_LIB_CTX *) = {
     oqshybkem_init_ecp, oqshybkem_init_ecx};
 extern const char *oqs_oid_alg_list[];
 
@@ -1506,8 +1503,8 @@ OQSX_KEY *oqsx_key_new(OSSL_LIB_CTX *libctx, char *oqs_name, char *tls_name,
         evp_ctx = OPENSSL_zalloc(sizeof(OQSX_EVP_CTX));
         ON_ERR_GOTO(!evp_ctx, err);
 
-        ret2 =
-            (init_kex_fun[primitive - KEY_TYPE_ECP_HYB_KEM])(tls_name, evp_ctx);
+        ret2 = (init_kex_fun[primitive - KEY_TYPE_ECP_HYB_KEM])(
+            tls_name, evp_ctx, libctx);
         ON_ERR_GOTO(ret2 <= 0 || !evp_ctx->keyParam || !evp_ctx->ctx, err);
 
         ret->numkeys = 2;
@@ -1539,7 +1536,7 @@ OQSX_KEY *oqsx_key_new(OSSL_LIB_CTX *libctx, char *oqs_name, char *tls_name,
         evp_ctx = OPENSSL_zalloc(sizeof(OQSX_EVP_CTX));
         ON_ERR_GOTO(!evp_ctx, err);
 
-        ret2 = oqsx_hybsig_init(bit_security, evp_ctx, tls_name);
+        ret2 = oqsx_hybsig_init(bit_security, evp_ctx, libctx, tls_name);
         ON_ERR_GOTO(ret2 <= 0 || !evp_ctx->ctx, err);
 
         ret->numkeys = 2;
@@ -1593,7 +1590,7 @@ OQSX_KEY *oqsx_key_new(OSSL_LIB_CTX *libctx, char *oqs_name, char *tls_name,
                 evp_ctx = OPENSSL_zalloc(sizeof(OQSX_EVP_CTX));
                 ON_ERR_GOTO(!evp_ctx, err);
 
-                ret2 = oqsx_hybsig_init(bit_security, evp_ctx, name);
+                ret2 = oqsx_hybsig_init(bit_security, evp_ctx, libctx, name);
                 ON_ERR_GOTO(ret2 <= 0 || !evp_ctx->ctx, err);
                 ret->oqsx_provider_ctx.oqsx_evp_ctx = evp_ctx;
                 ret->privkeylen_cmp[i] = ret->oqsx_provider_ctx.oqsx_evp_ctx
@@ -1809,6 +1806,7 @@ static int oqsx_key_gen_oqs(OQSX_KEY *key, int gen_kem) {
  * pubkey/privkey buffers; returned EVP_PKEY must be freed if not used
  */
 static EVP_PKEY *oqsx_key_gen_evp_key_sig(OQSX_EVP_CTX *ctx,
+                                          OSSL_LIB_CTX *libctx,
                                           unsigned char *pubkey,
                                           unsigned char *privkey, int encode) {
     int ret = 0, ret2 = 0, aux = 0;
@@ -1825,9 +1823,10 @@ static EVP_PKEY *oqsx_key_gen_evp_key_sig(OQSX_EVP_CTX *ctx,
     }
 
     if (ctx->keyParam)
-        kgctx = EVP_PKEY_CTX_new(ctx->keyParam, NULL);
+        kgctx = EVP_PKEY_CTX_new_from_pkey(libctx, ctx->keyParam, NULL);
     else
-        kgctx = EVP_PKEY_CTX_new_id(ctx->evp_info->nid, NULL);
+        kgctx = EVP_PKEY_CTX_new_from_name(
+            libctx, OBJ_nid2sn(ctx->evp_info->nid), NULL);
     ON_ERR_SET_GOTO(!kgctx, ret, -1, errhyb);
 
     ret2 = EVP_PKEY_keygen_init(kgctx);
@@ -1879,8 +1878,9 @@ static EVP_PKEY *oqsx_key_gen_evp_key_sig(OQSX_EVP_CTX *ctx,
                             privkeylen > (int)ctx->evp_info->length_private_key,
                         ret, -12, errhyb);
         // selftest:
-        EVP_PKEY *ck2 = d2i_PrivateKey(ctx->evp_info->keytype, NULL,
-                                       &privkey_enc2, privkeylen);
+        EVP_PKEY *ck2 =
+            d2i_PrivateKey_ex(ctx->evp_info->keytype, NULL, &privkey_enc2,
+                              privkeylen, libctx, NULL);
         ON_ERR_SET_GOTO(!ck2, ret, -14, errhyb);
         EVP_PKEY_free(ck2);
     }
@@ -1916,6 +1916,7 @@ static EVP_PKEY *oqsx_key_gen_evp_key_kem(OQSX_KEY *key, unsigned char *pubkey,
     unsigned char *pubkey_encoded = NULL;
     int idx_classic;
     OQSX_EVP_CTX *ctx = key->oqsx_provider_ctx.oqsx_evp_ctx;
+    OSSL_LIB_CTX *libctx = key->libctx;
 
     size_t pubkeylen = 0, privkeylen = 0;
 
@@ -1923,9 +1924,10 @@ static EVP_PKEY *oqsx_key_gen_evp_key_kem(OQSX_KEY *key, unsigned char *pubkey,
     unsigned char *privkey_sizeenc = key->privkey;
 
     if (ctx->keyParam)
-        kgctx = EVP_PKEY_CTX_new(ctx->keyParam, NULL);
+        kgctx = EVP_PKEY_CTX_new_from_pkey(libctx, ctx->keyParam, NULL);
     else
-        kgctx = EVP_PKEY_CTX_new_id(ctx->evp_info->nid, NULL);
+        kgctx = EVP_PKEY_CTX_new_from_name(
+            libctx, OBJ_nid2sn(ctx->evp_info->nid), NULL);
     ON_ERR_SET_GOTO(!kgctx, ret, -1, errhyb);
 
     ret2 = EVP_PKEY_keygen_init(kgctx);
@@ -1969,8 +1971,9 @@ static EVP_PKEY *oqsx_key_gen_evp_key_kem(OQSX_KEY *key, unsigned char *pubkey,
                             privkeylen > (int)ctx->evp_info->length_private_key,
                         ret, -12, errhyb);
         // selftest:
-        EVP_PKEY *ck2 = d2i_PrivateKey(ctx->evp_info->keytype, NULL,
-                                       &privkey_enc2, privkeylen);
+        EVP_PKEY *ck2 =
+            d2i_PrivateKey_ex(ctx->evp_info->keytype, NULL, &privkey_enc2,
+                              privkeylen, libctx, NULL);
         ON_ERR_SET_GOTO(!ck2, ret, -14, errhyb);
         EVP_PKEY_free(ck2);
     }
@@ -2009,8 +2012,9 @@ int oqsx_key_gen(OQSX_KEY *key) {
         ON_ERR_GOTO(ret, err_gen);
         ret = oqsx_key_gen_oqs(key, 1);
     } else if (key->keytype == KEY_TYPE_HYB_SIG) {
-        pkey = oqsx_key_gen_evp_key_sig(key->oqsx_provider_ctx.oqsx_evp_ctx,
-                                        key->pubkey, key->privkey, 1);
+        pkey =
+            oqsx_key_gen_evp_key_sig(key->oqsx_provider_ctx.oqsx_evp_ctx,
+                                     key->libctx, key->pubkey, key->privkey, 1);
         ON_ERR_GOTO(pkey == NULL, err_gen);
         ret = !oqsx_key_set_composites(key, 0);
         ON_ERR_GOTO(ret, err_gen);
@@ -2046,8 +2050,8 @@ int oqsx_key_gen(OQSX_KEY *key) {
             }
             if (get_oqsname_fromtls(name) == 0) {
                 pkey = oqsx_key_gen_evp_key_sig(
-                    key->oqsx_provider_ctx.oqsx_evp_ctx, key->comp_pubkey[i],
-                    key->comp_privkey[i], 0);
+                    key->oqsx_provider_ctx.oqsx_evp_ctx, key->libctx,
+                    key->comp_pubkey[i], key->comp_privkey[i], 0);
                 OPENSSL_free(name);
                 ON_ERR_GOTO(pkey == NULL, err_gen);
                 key->classical_pkey = pkey;
